package cn.com.ty.lift.business.annualinspection.controller;


import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.service.AnnualInspectionService;
import cn.com.ty.lift.business.library.dao.entity.model.response.LiftAnnualInspectionResponse;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.dao.entity.Region;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.business.push.service.PushUserService;
import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.model.PushMessage;
import cn.com.ty.lift.common.model.PushUserInfo;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.ty.lift.common.verify.Ver;
import cn.com.ty.lift.common.verify.Verifier;
import cn.com.ty.lift.common.verify.Verify;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 年检记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/annualinspection/annual-inspection")
public class AnnualInspectionController {

    private AnnualInspectionService     annualInspectionService;
    private LiftService                 liftService;
    private ProjectLiftRelevanceService projectLiftRelevanceService;
    private JmsMessagingTemplate        jmsMessagingTemplate;
    private PushUserService             pushUserService;

    private static final Map<String, String>     headerAlias = new HashMap<>();

    static {
        headerAlias.put("areaName", "区域");
        headerAlias.put("projectName", "项目");
        headerAlias.put("useCompanyCode", "梯号");
        headerAlias.put("registrationCode", "注册代码");
        headerAlias.put("devicePosition", "电梯位置");
        headerAlias.put("directorName", "区域主管");
        headerAlias.put("workerName", "维保工");
        headerAlias.put("planDate", "年检时间");
        headerAlias.put("checkGoverner", "是否检查限速器");
        headerAlias.put("loadInspection", "是否荷载试验");
        headerAlias.put("statusDesc", "状态");
    }

    /**
     * 根据id查询
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("findOne")
    @Verifier(fields = {"id"})
    public RestResponse findOne(@Ver @RequestBody InspectionRequest request) {
        InspectionResponse entity = annualInspectionService.infoById(request);
        return RestResponse.success(entity);
    }

    /**
     * 根据条件查询年检记录
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("page")
    @Verifier(fields = "mtCompanyId")
    public RestResponse page(@Ver @RequestBody InspectionRequest request) {
        IPage<InspectionResponse> pages = annualInspectionService.pageByCondition(request);
        long count = annualInspectionService.countByCondition(request);
        return RestResponse.success(CountPage.getCountPage(count, pages));
    }

    /**
     * 根据条件查询年检记录
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("pageWorker")
    @Verifier(fields = {"workerId"})
    public RestResponse pageWorker(@Ver @RequestBody InspectionRequest request) {
        IPage<InspectionResponse> pages = annualInspectionService.pageByCondition(request);
        return RestResponse.success(pages);
    }

    /**
     * 待确认年检计划, 根据电梯中的年检时间，提前45天通知
     */
    @PostMapping("toConfirm")
    @Verifier(fields = {"mtCompanyId"})
    public RestResponse toConfirm(@Ver @RequestBody InspectionRequest request){
        IPage<LiftAnnualInspectionResponse> pages = liftService.toConfirmAnnualInspection(request.getPage(), request.getMtCompanyId(), 45);
        return RestResponse.success(pages);
    }

    /**
     * status 状态（0 待完成；1 已完成；2 超期）
     * 1  维保工确认年检计划。停止信息推送
     * stepStatus : 1
     * stepName: 1
     */
    @PostMapping("confirm")
    @Verifier(fields = {"liftId","mtCompanyId","planDate","isCheckGoverner","loadInspectionSetting"})
    public RestResponse confirm(@Ver @RequestBody AnnualInspection entity){
        AnnualInspection sameConfirm = annualInspectionService.sameConfirm(entity.getLiftId(), entity.getMtCompanyId(), entity.getPlanDate());
        if(Objects.nonNull(sameConfirm)){
            return RestResponse.fail("年检计划已存在，创建于：" + sameConfirm.getCreateDate());
        }

        AnnualInspection current = annualInspectionService.findCurrentConfirm(entity.getLiftId(), entity.getMtCompanyId());
        if(Objects.nonNull(current) && Objects.nonNull(current.getPlanDate())){
            LocalDate deadline = current.getPlanDate().plusDays(DateUtils.daysOfYear());
            Verify.notTrue(deadline.isBefore(entity.getPlanDate()), "上次年检" + current.getPlanDate() + "，计划年检设置" + deadline + "之前有效");
        }

        entity.setStatus(0);
        entity.setStepStatus(1);
        entity.setStepName(1);
        boolean ai = annualInspectionService.saveOrUpdate(entity);
        return RestResponse.success(ai);
    }
    /**
     * 修改年检信息
     * <p>
     * status 状态（0 待完成；1 已完成；2 超期）
     * stepStatus  :年检阶段状态- 第一阶段 1-2，第二阶段 3，第三阶段 4-7，第四阶段 8-10
     * stepName : 当前能操作的阶段
     * <p>
     * 1  维保工确认年检计划
     * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
     *      文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
     * --------------------------------
     * 3  企业文员确认现场检验时间
     *      录入人员信息，政府质检，企业质检，联系电话
     * --------------------------------
     * 4	维保工上传检验结果, 如果有急修就关闭急修
     * 5	企业文员确认检验结果-合格 --> 不能算年检超期
     * 6	企业文员确认检验结果-整改 --> 回到第二阶段
     * 7	企业文员确认检验结果-不合格 --> 可以算年检超期
     * --------------------------------
     * 8	维保工上传年检报告和合格证
     * 9	企业文员确认年检报告和合格证-合格
     * 10	企业文员确认年检报告和合格证-不合格 -->停用电梯
     *
     * @param entity AnnualInspection
     * @return RestResponse
     */
    @PostMapping("update")
    @Verifier(fields = {"id","mtCompanyId","isCheckGoverner","loadInspectionSetting"})
    public RestResponse update(@Ver @RequestBody InspectionRequest entity){
        AnnualInspection old = annualInspectionService.findByMtCompanyId(entity.getId(), entity.getMtCompanyId());
        Verify.notNull(old, Verify.Inspection.confirm);
        Integer oldStepName = Objects.isNull(old.getStepName()) ? 1 : old.getStepName();
        Verify.isTrue(oldStepName == 1, "年检第一阶段才能修改");
        old.setIsCheckGoverner(entity.getIsCheckGoverner());
        old.setLoadInspectionSetting(entity.getLoadInspectionSetting());

        old.setStepStatus(1);
        old.setStepName(1);
        boolean ai = annualInspectionService.saveOrUpdate(old);
        return RestResponse.success(ai);
    }

    /**
     * 1 -> 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
     *      文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
     */
    @PostMapping("stepOne")
    @Verifier(fields = {"id","mtCompanyId","stepStatus","selfcheckReportImg","selfcheckDate"})
    public RestResponse stepOne(@Ver @RequestBody InspectionRequest entity){
        AnnualInspection old = annualInspectionService.findByMtCompanyId(entity.getId(), entity.getMtCompanyId());
        Verify.notNull(old, Verify.Inspection.confirm);
        Integer oldStepName = old.getStepName();
        Integer oldStepStatus = old.getStepStatus();
        Integer stepStatus = entity.getStepStatus();
        Verify.isTrue(oldStepName == 1, "请先完成确认操作再进行第一阶段");
        Verify.isTrue(oldStepStatus == 1 || oldStepStatus == 2, "请进行第一阶段操作");
        Verify.isTrue(stepStatus == 2, "请进行第一阶段操作");

        old.setSelfcheckReportImg(entity.getSelfcheckReportImg());
        old.setSelfcheckDate(entity.getSelfcheckDate());
        old.setStepStatus(entity.getStepStatus());

        old.setStepName(2);//流转到阶段2

        boolean ai = annualInspectionService.saveOrUpdate(old);
        return RestResponse.success(ai);
    }

    /**
     * 2 -> 3  企业文员确认现场检验时间
     *      录入人员信息，政府质检，企业质检，联系电话
     */
    @PostMapping("stepTwo")
    @Verifier(fields = {"id","mtCompanyId","checkDate","inspector","operator","telephone"})
    public RestResponse stepTwo(@Ver @RequestBody InspectionRequest entity){
        AnnualInspection old = annualInspectionService.findByMtCompanyId(entity.getId(), entity.getMtCompanyId());
        Verify.notNull(old, Verify.Inspection.confirm);
        Integer oldStepName = old.getStepName();
        Integer oldStepStatus = old.getStepStatus();
        Verify.isTrue(oldStepName == 1 || oldStepName == 2, "请先完成第一阶段再进行第二阶段");
        Verify.isTrue(oldStepStatus == 2 || oldStepStatus == 3, "请进行第二阶段操作");

        old.setCheckDate(entity.getCheckDate());
        old.setInspector(entity.getInspector());
        old.setOperator(entity.getOperator());
        old.setTelephone(entity.getTelephone());

        old.setStepStatus(3);
        old.setStepName(3);//流转到阶段3

        boolean ai = annualInspectionService.saveOrUpdate(old);
        if(ai){
            Region region = pushUserService.findRegionByAnnualInspection(old.getId(), old.getMtCompanyId());
            InspectionResponse info = annualInspectionService.infoById(entity);
            List<Long> userIds = new ArrayList<>();
            //区域主管
            if (Objects.nonNull(region)) {
                userIds.add(region.getUserId());
            }
            //电梯主要负责人
            ProjectLiftRelevance relevance = projectLiftRelevanceService.getOne(info.getMtCompanyId(), info.getLiftId());
            if (Objects.nonNull(relevance)) {
                userIds.add(relevance.getWorkerId());
            }
            if(!userIds.isEmpty()){
                List<PushUserInfo> pushUserInfos = pushUserService.listByUserIds(old.getMtCompanyId(), userIds);
                PushMessage.inspectionConfirmCheckDate(info.getProjectName(), info.getDevicePosition(), info.getRegistrationCode(), info.getCheckDate()).sendTokenOnPlatform(jmsMessagingTemplate, pushUserInfos);
            }
        }
        return RestResponse.success(ai);
    }

    /**
     * 3 -> 4	维保工上传检验结果
     * 5	企业文员确认检验结果-合格 --> 不能算年检超期
     * 6	企业文员确认检验结果-整改 --> 回到第二阶段
     *      (3  企业文员确认现场检验时间 录入人员信息，政府质检，企业质检，联系电话)
     * 7	企业文员确认检验结果-不合格 --> 可以算年检超期
     *  设置下次年检时间，是否检查限速器和是否荷载设置
     */
    @PostMapping("stepThree")
    @Verifier(fields = {"id","mtCompanyId","stepStatus","checkResultImg","projectId","nextInspectionTime","isCheckGoverner","loadInspectionSetting"})
    public RestResponse stepThree(@Ver @RequestBody InspectionRequest entity){
        AnnualInspection old = annualInspectionService.findByMtCompanyId(entity.getId(), entity.getMtCompanyId());
        Verify.notNull(old, Verify.Inspection.confirm);
        Integer oldStepName = old.getStepName();
        Integer oldStepStatus = old.getStepStatus();
        Integer stepStatus = entity.getStepStatus();
        Verify.isTrue(oldStepName == 2 || oldStepName == 3, "请先完成第二阶段再进行第三阶段");
        Verify.isTrue(oldStepStatus >= 3 && oldStepStatus <= 7, "请进行第三阶段操作");
        Verify.isTrue(stepStatus >= 4 && stepStatus <= 7, "请进行第三阶段操作");
        if(Objects.nonNull(old.getPlanDate())){
            LocalDate deadline = old.getPlanDate().plusDays(DateUtils.daysOfYear());
            Verify.notTrue(deadline.isBefore(entity.getNextInspectionTime()), "上次年检" + old.getPlanDate() + "，下次年检设置" + deadline + "之前有效");
        }

        old.setStepStatus(entity.getStepStatus());
        old.setCheckResultImg(entity.getCheckResultImg());
        old.setNextInspectionTime(entity.getNextInspectionTime());

        //企业文员确认检验结果-整改 --> 回到第二阶段
        if(stepStatus == 5 || stepStatus == 7){
            old.setStepName(4);//流转到阶段4
        } else if(stepStatus == 6){
            old.setStepStatus(3);
            old.setStepName(2);//转回阶段2
            old.setCheckResultImg("");
            old.setNextInspectionTime(LocalDate.of(1970, 1,1));
        }
        return annualInspectionService.stepThree(old, entity.getProjectId(), entity.getIsCheckGoverner(), entity.getLoadInspectionSetting());
    }

    /**
     * 5,7 -> 8	维保工上传年检报告和合格证
     * 9	企业文员确认年检报告和合格证-合格
     * 10	企业文员确认年检报告和合格证-不合格 -->停用电梯
     */
    @PostMapping("stepFour")
    @Verifier(fields = {"id","mtCompanyId","stepStatus","annualInspectionImg","certificateImgUrl"})
    public RestResponse stepFour(@Ver @RequestBody InspectionRequest entity){
        AnnualInspection old = annualInspectionService.findByMtCompanyId(entity.getId(), entity.getMtCompanyId());
        Verify.notNull(old, Verify.Inspection.confirm);
        Integer oldStepName = Objects.isNull(old.getStepName()) ? 3 : old.getStepName();
        Integer oldStepStatus =  Objects.isNull(old.getStepStatus()) ? 5 : old.getStepStatus();
        Integer stepStatus = entity.getStepStatus();
        Verify.isTrue(oldStepName == 3 || oldStepName == 4, "请先完成第三阶段再进行第四阶段");
        Verify.isTrue(oldStepStatus >= 5 && oldStepStatus <= 10, "请进行第四阶段操作");
        Verify.isTrue(stepStatus >= 8 && stepStatus <= 10, "请进行第四阶段操作");

        old.setStepStatus(entity.getStepStatus());
        old.setAnnualInspectionImg(entity.getAnnualInspectionImg());
        old.setCertificateImgUrl(entity.getCertificateImgUrl());

        if(stepStatus == 9){
            old.setStatus(1);//已完成
            old.setFinishTime(LocalDateTime.now());
        } else if(stepStatus == 10){
            old.setStatus(2);//超期
            old.setFinishTime(LocalDateTime.now());
        }
        old.setStepName(4);
        return annualInspectionService.stepFour(old);
    }

    /**
     * 其他角色获取自己关联的项目的年检任务
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("pageByUser")
    @Verifier(fields = {"userId"})
    public RestResponse pageByUser(@Ver @RequestBody InspectionRequest request) {
        IPage<InspectionResponse> pages = annualInspectionService.pageByUser(request);
        return RestResponse.success(pages);
    }

    /**
     * 确认年检计划，传递年检id，后台修改年检状态，停止信息推送
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
//    @PostMapping("confirm")
//    @Verifier(fields = {"id"})
//    public RestResponse confirm(@RequestBody InspectionRequest request) {
//        AnnualInspection entity = annualInspectionService.getById(request.getId());
//        Verify.notNull(entity);
//        entity.setStepStatus(Verify.Inspection.STATUS_CONFIRM);
//        entity.setStepName(Verify.Inspection.STEP_ONE);
//        boolean result = annualInspectionService.updateById(entity);
//        return RestResponse.success(result);
//    }

    /**
     * 上传自检报告，传递年检id，传递自检报告图片路径
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("selfcheck")
    @Verifier(fields = {"id", "selfcheckDate", "selfcheckReportImg"})
    public RestResponse selfcheck(@Ver @RequestBody InspectionRequest request) {
        AnnualInspection entity = annualInspectionService.getById(request.getId());
        Verify.notNull(entity);

        entity.setSelfcheckDate(request.getSelfcheckDate());
        entity.setSelfcheckReportImg(request.getSelfcheckReportImg());
        entity.setStepStatus(Verify.Inspection.STATUS_SELFCHECK);
        entity.setStepName(Verify.Inspection.STEP_ONE);

        boolean result = annualInspectionService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * 上传检验结果，传递年检id，传递检验报告图片路径
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("check")
    @Verifier(fields = {"id", "checkDate", "checkResultImg"})
    public RestResponse check(@Ver @RequestBody InspectionRequest request) {
        AnnualInspection entity = annualInspectionService.getById(request.getId());
        Verify.notNull(entity);

        entity.setCheckDate(request.getCheckDate());
        entity.setCheckResultImg(request.getCheckResultImg());
        entity.setStepStatus(Verify.Inspection.STATUS_CHECK_UPLOAD);
        entity.setStepName(Verify.Inspection.STEP_THREE);

        boolean result = annualInspectionService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * 上传年检报告和合格证，传递年检id，传递年检报告和合格证图片路径
     *
     * @param request InspectionRequest
     * @return RestResponse
     */
    @PostMapping("finish")
    @Verifier(fields = {"id", "certificateImgUrl", "annualInspectionImg"})
    public RestResponse finish(@Ver @RequestBody InspectionRequest request) {
        AnnualInspection entity = annualInspectionService.getById(request.getId());
        Verify.notNull(entity);

        entity.setCertificateImgUrl(request.getCertificateImgUrl());
        entity.setAnnualInspectionImg(request.getAnnualInspectionImg());
        entity.setStepStatus(Verify.Inspection.STATUS_CERTIFICATE_REPORT_UPLOAD);
        entity.setStepName(Verify.Inspection.STEP_FOUR);

        boolean result = annualInspectionService.updateById(entity);
        return RestResponse.success(result);
    }

    @PostMapping("export")
    @Verifier(fields = {"ids"})
    public void export(@Ver @RequestBody InspectionRequest request, HttpServletResponse response) {
        log.info("headerAlias: {}", headerAlias);
        //查询数据
        List<InspectionResponse> inspections = annualInspectionService.listByIdList(request);
        Verify.notNull(inspections);

        //压缩文件夹
        File baseDir = null;
        File zipFile = null;
        try {
            //获取根目录
            String export = ResourceUtils.getURL("classpath:export").getPath();
            String base = "inspection";
            String dir = StrUtil.join(File.separator, export, base, IdWorker.getIdStr());
            String zip = dir + ".zip";
            baseDir = new File(dir);
            zipFile = new File(zip);
            log.info("dir: {}", dir);
            log.info("zip: {}", zip);


            //生成excel文件
            String excel = File.separator + dir + File.separator + DateUtils.generateCode() + ".xlsx";
            log.info("生成excel: {}", excel);
            File excelFile = new File(excel);
            ExcelWriter writer = ExcelUtil.getBigWriter(excelFile, "年检记录表");
            //设置列别名
            headerAlias.forEach(writer::addHeaderAlias);
            //如果为true，则不设置alias的字段将不被输出，false表示原样输出
            writer.setOnlyAlias(true);
            writer.write(inspections, true);
//            writer.autoSizeColumnAll();
            writer.close();
            FileUtil.touch(excelFile);

            //处理图片类文件
            //windows现在已知的文件名非法字符有 \ / : * ? " < > |
            String illegalChars = "\\/:*?\"<>|";
            for (int i = 0; i < inspections.size(); i++) {
                InspectionResponse ins = inspections.get(i);
                String areaName = ins.getAreaName();
                String projectName = ins.getProjectName();
                String registrationCode = ins.getRegistrationCode();
                areaName = StrUtil.isNotEmpty(areaName) ? areaName.replace(illegalChars,"-") : "[区域]";
                projectName = StrUtil.isNotEmpty(projectName) ? projectName.replace(illegalChars,"-") : "[项目]";
                registrationCode = StrUtil.isNotEmpty(registrationCode) ? registrationCode.replace(illegalChars,"-") : "[电梯注册代码]";
                //图片文件夹 dir/i.AreaName-ProjectName-RegistrationCode/
                String imgDir = StrUtil.format("{}{}{}.{}-{}-{}{}", dir, File.separator, (i + 1), areaName, projectName, registrationCode, File.separator);
                downloadImage(ins.getSelfcheckReportImg(), imgDir, "1.自检报告.jpg");
                downloadImage(ins.getCheckResultImg(), imgDir, "2.检验结果.jpg");
                downloadImage(ins.getCertificateImgUrl(), imgDir, "3.合格证.jpg");
                downloadImage(ins.getAnnualInspectionImg(), imgDir, "4.年检报告.jpg");
            }

            //压缩文件夹
            log.info("生成zipfile: {}", zip);
            ZipUtil.zip(dir, zip);
        } catch (Exception e) {
            FileUtil.del(zipFile);
            FileUtil.del(baseDir);
            log.error("生成文件失败", e);
            throw Verify.verifyException("生成文件失败");
        }
        //下载压缩文件
        if (FileUtil.exist(zipFile)) {
            // 设置强制下载不打开, ZIP的application/x-zip-compressed
            response.setContentType("application/force-download");
            // 设置文件名
            response.addHeader("Content-Disposition", "attachment;fileName=" + zipFile.getName());
            byte[] buffer = new byte[1024];
            try (FileInputStream fis = new FileInputStream(zipFile);
                 BufferedInputStream bis = new BufferedInputStream(fis)) {
                OutputStream os = response.getOutputStream();
                int i;
                while ((i = bis.read(buffer)) != -1) {
                    os.write(buffer, 0, i);
                }
            } catch (Exception e) {
                log.error("下载文件失败", e);
                throw Verify.verifyException("下载文件失败");
            }finally {
                //删除临时根目录和zip文件
                FileUtil.del(zipFile);
                FileUtil.del(baseDir);
            }
        }
    }

    //根据图片URL生成图片文件
    private void downloadImage(String imgUrl, String imgDir, String imgName) {
        if (StrUtil.isNotEmpty(imgUrl)) {
            log.info("imgUrl: {}", imgUrl);
            String filename = imgDir + imgName;
            log.info("{}: {}", imgName, filename);
            HttpUtil.downloadFile(imgUrl, filename);
        }
    }

    @PostMapping("pageByLift")
    @Verifier(fields = {"mtCompanyId", "liftId", "projectId"})
    public RestResponse pageByLift(@Ver @RequestBody InspectionRequest request){
        IPage<InspectionResponse> pages = annualInspectionService.pageByLift(request);
        return RestResponse.success(pages);
    }
}
