package cn.com.ty.lift.enterprise.custom.controller;

import cn.com.ty.lift.common.verify.Verify;
import cn.com.ty.lift.enterprise.custom.dao.entity.PropertyCompany;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyAndPropertyContactReq;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyReq;
import cn.com.ty.lift.enterprise.custom.service.impl.PropertyCompanyServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 客户接口(物业公司)(甲方公司) 前端控制器
 *
 * @author wang-hai-cheng
 * @since 2019/11/27
 */
@RestController
@RequestMapping("/property-company")
@AllArgsConstructor
public class PropertyCompanyController {
    private final PropertyCompanyServiceImpl propertyCompanyService;

    /**
     * 根据条件查询客户列表
     *
     * @param request 客户名称或客户地址或客户邮寄地址或客户联系方式,页码,页大小,维保公司id
     * @return RestResponse
     * @since 2019/12/3 8:51
     */
    @PostMapping("/list")
    public RestResponse propertyCompanies(@RequestBody PropertyCompanyReq request) {
        Verify.id(request.getMtCompanyId());
        Verify.notNull(request.getPageNum());
        Verify.notNull(request.getPageSize());
        return propertyCompanyService.propertyCompanies(request);
    }

    /**
     * 根据id获取客户详细信息
     *
     * @param id [客户id]
     * @return RestResponse
     * @since 2019/11/27 11:36
     */
    @GetMapping("/{id}")
    public RestResponse propertyCompany(@PathVariable Long id) {
        Verify.id(id);
        PropertyCompany byId = propertyCompanyService.getById(id);
        if (byId == null) {
            return RestResponse.success();
        }
        return RestResponse.success(byId);
    }

    /**
     * 新增客户
     *
     * @param request [客户]
     * @return RestResponse
     * @since 2019/11/27 11:45
     */
    @PostMapping("/add")
    public RestResponse add(@RequestBody PropertyCompanyAndPropertyContactReq request) {
        return propertyCompanyService.saveAll(request);
    }

    /**
     * 更新客户信息
     *
     * @param propertyCompany [客户]
     * @return RestResponse
     * @since 2019/11/27 11:45
     */
    @PostMapping("/update")
    public RestResponse update(@RequestBody PropertyCompany propertyCompany) {
        Verify.id(propertyCompany.getId());
        if (propertyCompanyService.updateById(propertyCompany)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    /**
     * 删除客户
     *
     * @param propertyCompany [客户id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     * @since 2019/11/27 11:46
     */
    @PostMapping("/delete")
    public RestResponse delete(@RequestBody PropertyCompany propertyCompany) {
        return propertyCompanyService.delete(propertyCompany.getId());
    }

    /**
     * 项目模块用，根据维保公司id查询客户公司列表
     * @param propertyCompany [mtCompanyId] 维保公司id
     * @return 客户公司列表
     */
    @PostMapping("/project/customs")
    public RestResponse customs(@RequestBody PropertyCompany propertyCompany) {
        List<PropertyCompany> propertyCompanies = propertyCompanyService.list(new QueryWrapper<PropertyCompany>()
                .select("id", "name")
                .eq("mt_company_id", propertyCompany.getMtCompanyId())
                .ne("status", 0));
        if (propertyCompanies.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(propertyCompanies);
    }

    /**
     * 项目模块，新增客户
     *
     * @since 2019/12/16 17:08
     */
    @PostMapping("/project/custom/add")
    public RestResponse customAdd(@RequestBody PropertyCompany propertyCompany) {
        propertyCompany.setStatus(2);
        if (propertyCompanyService.save(propertyCompany)) {
            return RestResponse.success(propertyCompany.getId(), "成功");
        }
        return RestResponse.fail();
    }

    @PostMapping("/export")
    public void export(@RequestBody PropertyCompany propertyCompany, HttpServletResponse response) {
        propertyCompanyService.export(propertyCompany, response);
    }
}